
recodificar <- function(z) {
  z[z == "101"] <- "S"
  z[z == "102"] <- "S"
  z[!(z %in% c("S"))] <- "N"
  z
}
plot_roc_curve <- function(rf, test, colores){
  require(ROCR)
  # Calcular la probabilidad de que una nueva observación pertenezca a una de las clases
  prediccion <- predict(rf, test, type="prob")
  # Clases
  classes <- levels(test$solucion_bin)
  # Inicialización AUC
  auc.perf <- c()
  for (i in 1:length(classes))
  {
    # Cargar valores que pertenecen a la clase i
    valores <- ifelse(test$solucion_bin==classes[i],1,0)
    # Predecir y performace
    pred <- prediction(prediccion[,i], valores)
    perf <- performance(pred, "tpr", "fpr")
    # Pintar
    if(i==1){
      plot(perf, main="Curva ROC", col=colores[i])
    } else {
      plot(perf, main="Curva ROC", col=colores[i],add=TRUE)
    }
    # Calcular AUC
    auc.perf <- c(auc.perf, performance(pred, measure = "auc")@y.values[[1]])
  }
  auc.perf
}
obtener_precision <- function(rf, test){
  t_pred = predict(rf, test)
  print(table(observed=test$solucion_bin, predicted=t_pred))
  confMat <- table(test$solucion_bin, t_pred)
  accuracy <- sum(diag(confMat))/sum(confMat)
  accuracy
}
########################


nombre_fichero <- "/home/garcia/Escritorio/documento/DAPFSP/rendimiento_modelos/resultados/resultados.rds"
resultados <- readRDS(nombre_fichero)
require(forcats)
resultados$solucion_bin<- fct_relabel(factor(resultados$estado_cplex), recodificar)
resultados$t_disponible <- as.factor(resultados$t_disponible)
resultados$cpu_disponible <- as.factor(resultados$cpu_disponible)
resultados$n <- as.factor(resultados$n)
resultados$m <- as.factor(resultados$m)
resultados$f <- as.factor(resultados$f)
resultados$t <- as.factor(resultados$t)

entrenamiento <- resultados[resultados$instancia!=1,]
test <- resultados[resultados$instancia==1,]
require(caret)
require(ggplot2)
require(randomForest)
require(doMC)
registerDoMC(detectCores() - 1)
require(devtools)
require(reprtree) # install_github('araastat/reprtree')
set.seed(4)

ntree_tests <- c(1:10,seq(10,20,2),seq(25,50,5))

resultados<-c()
modelos<-list()

for (i in 1:length(ntree_tests)){
  modelos[[i]]<-randomForest(solucion_bin ~ n + m + f + t + t_disponible + cpu_disponible,
                                  data=entrenamiento, importance=TRUE,  ntree=ntree_tests[i])
  
  resultados[i]<-obtener_precision(modelos[[i]], test) 
  plot(ntree_tests[1:i], resultados[1:i], type="o")
}

plot(ntree_tests, resultados, type="o", xlab="Número de árboles", ylab="Precisión", xaxt="n")
abline(v=5, col="red", lty=2)
abline(h=0.9907407, col="orange", lty=2)
axis(1,at=c(1,5))
axis(1,at=ntree_tests[-c(1:9)])
axis(1,at=ntree_tests, labels=rep("",22))

rf_clasificador <- modelos[[5]]
varImpPlot(rf_clasificador, main="")
title("Importancia de los factores", line=-0.5)

obtener_precision(rf_clasificador, test)
plot_roc_curve(rf_clasificador, test, c("#F8766D","#619CFF"))
tree <- getTree(rf_clasificador,1,labelVar=TRUE)
tree_text <- reprtree:::as.tree(tree, rf_clasificador)
graf<-reprtree:::plot.getTree(rf_clasificador, k=1, main="Árbol de decisión Random Forest")



#        predicted
#observed   S   N
#S 287   3
#N   1 141
#[1] 0.9259259

#Call:
#randomForest(formula = solucion_bin ~ n + m + f + t + t_disponible +      cpu_disponible, data = entrenamiento, importance = TRUE,      ntree = ntree_tests[i]) 
#Type of random forest: classification
#Number of trees: 5
#No. of variables tried at each split: 2
#
#OOB estimate of  error rate: 4.74%
#Confusion matrix:
#  S   N class.error
#S 1012  21  0.02032914
#N   53 476  0.10018904