setwd("~/Escritorio/TFG/documento/DAPFSP/rendimiento_modelos/codigo")
setwd("~/Escritorio/codigo")
require(Matrix)

# Capítulo 3: 

  # Parámetros de ejecución

  n <- c(4,8,12,16)  # Número de componentes a procesar
  m <- c(2,4,6)   # Número de máquinas 
  f <- c(2,3,4)  # Número de fábricas idénticas
  t <- c(2,3,4) # Número de productos finales
  
  limite_tiempo <- as.double(c(60*60, 10*60))  # Limite de tiempo, en segundos, para resolver CPLEX
  limite_nucleos <- as.integer(c(4,3)) # Límite de nucleos a usar CPLEX para resolver
  
  instancias_each <- 5  # Instancias generadas por problemas
  nombre_fichero <- "temp.lp"  # Archivo temporal que se genera con el modelo a ejecutar en CPLEX
  
  nombre_resultados <- "resultados.rds"   # Archivo con los resultados de procesamiento 
                                          # Parámetros de ejecución + Tiempo de ejecución de CPLEX + 
                                          # + Estado del solver al acabar su ejecución (MIP Problem status codes) 
                                          # + GAP final
  
  generar_datos <- 0  # 1 si se desea que se generen los datos
  
    entero_P <- 1  # 1 si se desea que los Pij generados sean enteros
    entero_PA <- 1 # 1 si se desea que los PAs generados sean enteros
  
    inic_P <- 1 # Rango de inicio de los Pij generados
    fin_P <- 99 # Rango final de los Pij generados
  
    inic_PA <- 1 # Rango de inicio de los PAs generados
    fin_PA <- 66 # Rango final de los PAs generados
  
  reanudar <- 1900
  
  
  source("libreria.R")   # Librería
  
  if(generar_datos == 1){
    crear_datos_DAPFSP(inic_P, fin_P, inic_PA, fin_PA, n, m, f, t, instancias_each, entero_P, entero_PA)
  }
  
  if(reanudar==0){
    # Construímos un data.frame para almacenar los resultados 
    tamaño_resultados <- length(n) * length(m) * length(f) * length(t) * instancias_each * length(limite_tiempo) * length(limite_nucleos)
    resultados <- data.frame( n=rep(0,tamaño_resultados), m=rep(0,tamaño_resultados), f=rep(0,tamaño_resultados), t=rep(0,tamaño_resultados),
                            instancia=rep(0,tamaño_resultados), t_disponible=rep(0,tamaño_resultados), cpu_disponible=rep(0,tamaño_resultados), 
                            t_cplex=rep(0,tamaño_resultados), estado_cplex=rep(0,tamaño_resultados), gap_cplex=rep(0,tamaño_resultados) )
    contador <- 1  # Contador para rellenar el data.frame secuencialmente
  } else{
    resultados <- readRDS(nombre_resultados)
  }
  
  # Inicio Solver
  for(a in 1:length(n)) {
    for(b in 1:length(m)) {
      for(c in 1:length(f)) {
        for(dd in 1:length(t)){
          for(d in 1:instancias_each){
            
            if(contador>reanudar){ 
              print(sprintf("Procesando %d componentes en %d máquinas sobre %d fábricas para producir %d productos", n[a], m[b], f[c], t[dd] ) )
              print(sprintf("Instancia %d de %d", d, instancias_each))
          
              generacion_modelo_DAPFSP(nombre_fichero, n[a], f[c], m[b], t[dd], d)
          
              for(e in 1:length(limite_tiempo)){
                for(ee in 1:length(limite_nucleos)){
              
                  print(sprintf("Características de la ejecución: Disponible %d segs y %d núcleos", limite_tiempo[e], limite_nucleos[ee] ) )
                  salida_cplex<-resolver_CPLEX(nombre_fichero, limite_tiempo[e], limite_nucleos[ee])
              
                  resultados[contador,] = c(n[a], m[b], f[c], t[dd], d, limite_tiempo[e], limite_nucleos[ee], salida_cplex)
                  print(resultados[contador,])
                  contador <- contador +1
                }
              }
              # Guardar resultados
              saveRDS(resultados, file=nombre_resultados)
            }
            else{ contador <- contador +length(limite_tiempo)+length(limite_nucleos) }
          }
        }
      }
    }
  }
  

  
  