crear_fichero <- function(nombre_fichero){
  sink(nombre_fichero)
  cat("\\ENCODING=ISO-8859-1", "\n")
  # "\" es un carácter especial. De la documentación de R: "reserved to escape the character that follows it."
  sink()
  
  
  sink(nombre_fichero, append=TRUE)
  cat("\n")
  cat("Minimize", "\n")
  cat(" tiempo_maximo_completacion: +Cmax", "\n")
  cat("Subject To", "\n")
  sink()
  
}

crear_datos_DAPFSP <-function(rango_inic_P, rango_fin_P, rango_inic_PA, rango_fin_PA, n, m, f, t, instancias_each, entero_P, entero_PA){
  
  require(Matrix)
  
  for(a in 1:length(n)){
    for(b in 1:length(m)){
      for(c in 1:length(f)){
        for(d in 1:length(t))
          for(e in 1:instancias_each){
            
            if(entero_P == 1){
              P <- matrix( sample(rango_inic_P:rango_fin_P, n[a] * m[b], replace=T), nrow=n[a])
            }
            if(entero_P == 0){
              P <- matrix( runif(rango_inic_P:rango_fin_P, n[a] * m[b]), nrow=n[a])
            }
            
            if(entero_PA == 1){
              PA <- sample(rango_inic_PA:rango_fin_PA, t[d], replace=T)
            }
            if(entero_PA == 0){
              PA <- runif(rango_inic_PA:rango_fin_PA, t[d])
            }
            
            G <- sparseMatrix( sample(1:n[a], replace=F), 
                               sample( rep( 1:t[d], 
                                            each = sample( n[a]*t[d]:(n[a]*t[d])^2, size=1 )
                                            ),
                                       size=n[a], 
                                       replace=F
                                     ),
                               dims = c(n[a], t[d])
                             )
            
            save(P,PA,G, file=sprintf("%d_%d_%d_%d_%d.rda", n[a], m[b], f[c], t[d], e))
        }
      }
    }
  }

}

resolver_CPLEX <- function(nombre_fichero, limite_tiempo, limite_nucleos){
  
  # Inicialización
    require(cplexAPI)
    env <- openEnvCPLEX()
    lp <- initProbCPLEX(env)
    setIntParmCPLEX(env, CPX_PARAM_SCRIND, CPX_ON)
  
  # Configuración de opciones
    setDblParmCPLEX(env, CPX_PARAM_TILIM, limite_tiempo)     # Límite de tiempo de ejecución
    setIntParmCPLEX(env, CPX_PARAM_THREADS, limite_nucleos)   # Límite de núcleos de ejecución
    
  # Solver
    readCopyProbCPLEX(env, lp, nombre_fichero)
    start<-getTimeCPLEX(env)
    mipoptCPLEX(env, lp)
    final<-getTimeCPLEX(env)
    
    estado <- getStatCPLEX(env, lp)
    gap <- getMIPrelGapCPLEX(env,lp)
    if(class(gap) == "cplexError"){
      # Significa que se ha producido un error en la ejecución (INFEASIBLE, ABORT...). No hay GAP
      gap <- NA
    }
    
  # Limpieza (Evitar problemas con el garbage collector)
    delProbCPLEX(env, lp)
    closeEnvCPLEX(env)
    rm(env)
    rm(lp)
  
  c(final-start, estado, gap)
}

generacion_modelo_DAPFSP <-function(nombre_fichero, num_componentes, num_fabricas, num_maquinas, num_productos, d){
  
  crear_fichero(nombre_fichero)
  
  componentes <- 1:num_componentes
  maquinas <- 1:num_maquinas
  fabricas <- 1:num_fabricas
  productos <- 1:num_productos
  
  # Leyendo datos
  load(sprintf("%d_%d_%d_%d_%d.rda", num_componentes, num_maquinas, num_fabricas, num_productos, d) )
  
  # Conexión con el fichero temporal
  sink(nombre_fichero, append=TRUE)
  
  # Restricción b. etiquetada como UnComponenteUnaPosicion
  for (j in componentes){
    cat( sprintf(" UnComponenteUnaPosicion_%d: +X_kdummy_%d", j, j) )
    
    for (k in componentes){
      if(j>k || j<k){
        cat( sprintf(" +X_%d_%d",k,j) )        
      }
    }
    cat(" = 1 \n")
  }
  
  # Restricción c. etiquetada como DefX
  for (k in componentes){
    cat( sprintf(" DefX_%d: +X_jdummy_%d", k,k) )
    
    for (j in componentes){
      if(j>k || j<k){
        cat( sprintf(" +X_%d_%d",k, j) )         
      }
    }
    cat(" <= 1 \n")
  }
  
  
  # Restricción d.1 etiquetada como Separadores1
  cat(" Separadores1: ")
  for (j in componentes){
    cat( sprintf(" +X_kdummy_%d", j) )
  }
  cat(" = ", num_fabricas, "\n")
  
  
  # Restricción d.2 etiquetada como Separadores2
  cat(" Separadores2: ")
  for (k in componentes){
    cat( sprintf(" +X_jdummy_%d", k) )
  }
  cat(" = ", num_fabricas-1, "\n")  
  
  
  
  # Restricción e. etiquetada como defprecedencia
  for (k in componentes){
    if(k<num_componentes){
      for(j in componentes){
        if(j>k){
          cat( sprintf(" defprecedencia_%d_%d: ",k,j), sprintf(" +X_%d_%d +X_%d_%d",k,j,j,k), " <= 1\n" )
        }
      }
    }
  }
  
  # Restricción f.1 etiquetada como defcompletacion0
  for(j in componentes){
    cat( sprintf(" defcompletacion0_%d: ", j), sprintf(" +C_%d_%d ", j, 1), " >= +", P[j,1], "\n" )
  }
  
  # Restricción f.2 etiquetada como defcompletacion1
  for(j in componentes){
    for(i in maquinas){
      if(i >1){
        cat( sprintf(" defcompletacion1_%d_%d: ", j, i), sprintf(" +C_%d_%d -C_%d_%d ", j, i, j, i-1), " >= +", P[j,i], "\n" )
      }
    }
  }
  
  M <- sum(P)+sum(PA)
  
  # Restricción f.3 etiquetada como defcompletacion2
  for(j in componentes){
    for(i in maquinas){
      cat( sprintf(" defcompletacion2_%d_%d: ", j, i), sprintf(" +C_%d_%d ", j, i), -M, sprintf(" X_kdummy_%d", j)   )
      cat(">= ", -M+P[j,i], "\n" )
    }
  } 
  
  # Restricción f.4 etiquetada como defcompletacion3
  for(k in componentes){
    for(j in componentes){
      for(i in maquinas){
        if(j>k || j<k){
          
          cat( sprintf(" defcompletacion3_%d_%d_%d: ", k, j, i), sprintf(" +C_%d_%d -C_%d_%d ", j, i, k,i))
          cat( " -", M, sprintf(" X_%d_%d", k,j) )
          cat(">=", -M+P[j,i], "\n" )
        }
      }
    }
  }
  
  # Restricción g. etiquetada como UnProductoUnaPosicion
  for (s in productos){
    cat( sprintf(" UnProductoUnaPosicion%d: +Y_ldummy_%d", s, s) )
    
    for (l in productos){
      if(l>s || l<s){
        cat( sprintf(" +Y_%d_%d",l,s) )        
      }
    }
    cat(" = 1 \n")
  }
  
  # Restricción h. etiquetada como DefY
  for (l in productos){
    cat( sprintf(" DefY_%d: ", l) )
    
    for (s in productos){
      if(l>s || l<s){
        cat( sprintf(" +Y_%d_%d",l, s) )         
      }
    }
    cat(" <= 1 \n")
  }
  
  
  # Restricción i. etiquetada como defprecedencia_pr
  for (l in productos){
    if(l<num_componentes){
      for(s in productos){
        if(s>l){
          cat( sprintf(" defprecedencia_pr_%d_%d: ",l,s), sprintf(" +Y_%d_%d +Y_%d_%d",l,s,s,l), " <= 1\n" )
        }
      }
    }
  }
  
  # Restricción j.1 etiquetada como defcompletacion4
  for(s in productos){
    for(j in componentes){
      cat( sprintf(" defcompletacion4_%d_%d: ", s, j), sprintf(" +CA_%d", s))
      
      if(G[j,s]==1){
        cat(sprintf(" -C_%d_%d", j,num_maquinas))
      }         
      
      cat(" >= +", PA[s], "\n" )
      
    }
  }
  
  # Restricción j.2 etiquetada como defcompletacion5
  for(s in productos){
    
    cat( sprintf(" defcompletacion5_%d: ", s), sprintf(" +CA_%d", s))
    cat( " -", M, sprintf(" Y_ldummy_%d", s) )
    cat(">=", -M+PA[s], "\n" )
    
  }  
  
  # Restricción j.3 etiquetada como defcompletacion6
  for(s in productos){
    for(l in productos){
      if(l>s || l<s){
        
        cat( sprintf(" defcompletacion6_%d_%d: ", s,l), sprintf(" +CA_%d -CA_%d", s,l))
        cat( " -", M, sprintf(" Y_%d_%d", l,s) )
        cat(">=", -M+PA[s], "\n" )     
        
      }
    }
  }    
  
  # Restricción k. etiquetada como defCmax
  for (s in productos){
    cat( sprintf(" defCmax_%d: +Cmax - CA_%d >= 0\n", s,s) )
  }
  
  
  
  # Bounds
  cat("Bounds", "\n", " Cmax >= 0", "\n")  
  
  for(j in componentes){
    for(i in maquinas){
      cat( sprintf("C_%d_%d >= 0",j, i), "\n" )
    }
  }
  
  for(s in productos){
    cat( sprintf("CA_%d >= 0",s), "\n" )
  }
  
  
  
  # Binary
  cat("Binary", "\n")  
  
  for(j in componentes){
    
    cat( sprintf("X_kdummy_%d",j), "\n" )
    
    for(k in componentes){
      
      if(j==1){
        cat( sprintf("X_jdummy_%d",k), "\n" )
      }
      
      if(k>j || k<j){
        cat( sprintf("X_%d_%d",k,j), "\n" )
      }
      
    }
  }
  
  
  for(s in productos){
    
    cat( sprintf("Y_ldummy_%d",s), "\n" )
    
    for(l in productos){
      
      if(l>s || l<s){
        cat( sprintf("Y_%d_%d",l,s), "\n" )
      }
      
    }
  }
  
  
  cat("End")
  sink()
}
