setwd("~/Escritorio/TFG/documento/DPFSP/rendimiento_modelos/codigo")

# Capítulo 2: 

  # Parámetros de ejecución

  n <- c(4,7,10,13)  # Número de trabajos a procesar
  m <- c(2,4,6)   # Número de máquinas 
  f <- c(2,3,4)  # Número de fábricas idénticas

  limite_tiempo <- as.double(c(30*60, 5*60))  # Limite de tiempo, en segundos, para resolver CPLEX
  limite_nucleos <- as.integer(c(4,3)) # Límite de nucleos a usar CPLEX para resolver
  
  instancias_each <- 5  # Instancias generadas por problemas
  nombre_fichero <- "temp.lp"  # Archivo temporal que se genera con el modelo a ejecutar en CPLEX
  
  modelo_ejecutar <- 6  # Número de modelo a ejecutar 1:6
  nombre_resultados <- "resultados6.rds"  # Archivo con los resultados de procesamiento 
                                          # Parámetros de ejecución + Tiempo de ejecución de CPLEX + 
                                          # + Estado del solver al acabar su ejecución (MIP Problem status codes) 
                                          # + GAP final
  
  generar_P <- 0  # 1 si se desea que se generen los Pij
  entero_P <- 1  # 1 si se desea que los Pij generados sean enteros
  
  inic_P <- 1 # Rango de inicio de los Pij generados
  fin_P <- 99 # Rango final de los Pij generados
  
  reanudar <-0
  
  
  source("libreria.R")   # Librería
  
  if(generar_P == 1){
    crear_P(inic_P, fin_P, n, m, f, instancias_each, entero_P)  # Generamos los P_ij
  }
  
  
  # Construímos un data.frame para almacenar los resultados 
  tamaño_resultados <- length(n) * length(m) * length(f) * instancias_each * length(limite_tiempo) * length(limite_nucleos)
  resultados <- data.frame( n=rep(0,tamaño_resultados), m=rep(0,tamaño_resultados), f=rep(0,tamaño_resultados), 
                            instancia=rep(0,tamaño_resultados), t_disponible=rep(0,tamaño_resultados), cpu_disponible=rep(0,tamaño_resultados), 
                            t_cplex=rep(0,tamaño_resultados), estado_cplex=rep(0,tamaño_resultados), gap_cplex=rep(0,tamaño_resultados) )
  contador <- 1  # Contador para rellenar el data.frame secuencialmente

  
  # Inicio Solver
  for(a in 1:length(n)) {
    for(b in 1:length(m)) {
      for(c in 1:length(f)) {
        for(d in 1:instancias_each){
          if(contador>reanudar){ 
          print(sprintf("Procesando %d trabajos en %d máquinas sobre %d fábricas", n[a], m[b], f[c] ) )
          print(sprintf("Instancia %d de %d", d, instancias_each))
          
          
          eval(parse( text = sprintf("generacion_modelo_%d(nombre_fichero, n[a], f[c], m[b], d)", modelo_ejecutar) ))

          
          for(e in 1:length(limite_tiempo)){
            for(ee in 1:length(limite_nucleos)){
              
              print(sprintf("Características de la ejecución: Disponible %d segs y %d núcleos", limite_tiempo[e], limite_nucleos[ee] ) )
              salida_cplex<-resolver_CPLEX(nombre_fichero, limite_tiempo[e], limite_nucleos[ee])
              
              resultados[contador,] = c(n[a], m[b], f[c], d, limite_tiempo[e], limite_nucleos[ee], salida_cplex)
              print(resultados[contador,])
              contador <- contador +1
            }
          }
          # Guardar resultados
          saveRDS(resultados, file=nombre_resultados)
          }
          else{ contador <- contador +length(limite_tiempo)+length(limite_nucleos) }
        }
      }
    }
  }
  

  
  