crear_fichero <- function(nombre_fichero){
  sink(nombre_fichero)
  cat("\\ENCODING=ISO-8859-1", "\n")
  # "\" es un carácter especial. De la documentación de R: "reserved to escape the character that follows it."
  sink()
  
  
  sink(nombre_fichero, append=TRUE)
  cat("\n")
  cat("Minimize", "\n")
  cat(" tiempo_maximo_completacion: +Cmax", "\n")
  cat("Subject To", "\n")
  sink()
  
}

crear_P <-function(rango_inic, rango_fin, n, m, f, instancias_each, entero){
  
  if(entero == 1)
  {
    for(a in 1:length(n)) {
      for(b in 1:length(m)) {
        for(c in 1:length(f)) {
          for(d in 1:instancias_each){
            
            P <- matrix( sample(rango_inic:rango_fin, n[a] * m[b], replace=T), nrow=n[a])
            saveRDS(P, file=sprintf("%d_%d_%d_%d.rds", n[a], m[b], f[c], d) )
            
          }
        }
      }
    }
  }
  
  if(entero == 0)
  {
    for(a in 1:length(n)) {
      for(b in 1:length(m)) {
        for(c in 1:length(f)) {
          for(d in 1:instancias_each){
            
            P <- matrix( runif(rango_inic:rango_fin, n[a] * m[b]), nrow=n[a])
            saveRDS(P, file=sprintf("%d_%d_%d_%d.rds", n[a], m[b], f[c], d) )
            
          }
        }
      }
    }
    
  }
  

}

resolver_CPLEX <- function(nombre_fichero, limite_tiempo, limite_nucleos){
  
  # Inicialización
    require(cplexAPI)
    env <- openEnvCPLEX()
    lp <- initProbCPLEX(env)
    setIntParmCPLEX(env, CPX_PARAM_SCRIND, CPX_ON)
  
  # Configuración de opciones
    setDblParmCPLEX(env, CPX_PARAM_TILIM, limite_tiempo)     # Límite de tiempo de ejecución
    setIntParmCPLEX(env, CPX_PARAM_THREADS, limite_nucleos)   # Límite de núcleos de ejecución
    
  # Solver
    readCopyProbCPLEX(env, lp, nombre_fichero)
    start<-getTimeCPLEX(env)
    mipoptCPLEX(env, lp)
    final<-getTimeCPLEX(env)
    
    estado <- getStatCPLEX(env, lp)
    gap <- getMIPrelGapCPLEX(env,lp)
    if(class(gap) == "cplexError"){
      # Significa que se ha producido un error en la ejecución (INFEASIBLE, ABORT...). No hay GAP
      gap <- NA
    }
    
  # Limpieza (Evitar problemas con el garbage collector)
    delProbCPLEX(env, lp)
    closeEnvCPLEX(env)
    rm(env)
    rm(lp)
  
  c(final-start, estado, gap)
}

generacion_modelo_1 <-function(nombre_fichero, num_trabajos, num_fabricas, num_maquinas, d){
  
  crear_fichero(nombre_fichero)
  
  trabajos <- rep(1:num_trabajos)
  maquinas <- rep(1:num_maquinas)
  fabricas <- rep(1:num_fabricas)
  
  # Leyendo Pij
  P<-readRDS( file=sprintf("%d_%d_%d_%d.rds", num_trabajos, num_maquinas, num_fabricas, d) )
  
  # Conexión con el fichero temporal
  sink(nombre_fichero, append=TRUE)
  
  # Restricción b. etiquetada como UnTrabajoUnaFabrica
  for (j in trabajos){
    cat( sprintf(" UnTrabajoUnaFabrica_%d:", j) )
    for (f in fabricas){
      cat( sprintf(" +Y_%d_%d",j, f) )
    }
    cat(" = 1 \n")
  }
  
  # Restricción c. etiquetada como UnTrabajoUnaPosicion
  for (j in trabajos){
    cat( sprintf(" UnTrabajoUnaPosicion_%d:", j) )
    for (f in fabricas){
      
      for (k in trabajos){
        if(j>k || j<k){
          cat( sprintf(" +X_%d_%d_%d",k, j, f) )         
        }
      }
      cat( sprintf(" +Xdummy_%d_%d", j, f) )  
    }
    cat(" = 1 \n")
  }
  
  
  # Restricción d. etiquetada como RelacionXeY
  for (j in trabajos){
    for (f in fabricas){
      
      cat( sprintf(" RelacionXeY_%d_%d:", j,f) )
      for (k in trabajos){
        if(j>k || j<k){
          cat( sprintf(" +X_%d_%d_%d +X_%d_%d_%d",k,j,f,j,k,f) )
        }
      }
      cat( sprintf(" -2 Y_%d_%d",j,f) )
      cat(" <= 0", "\n")
    }
  }
  
  # Restricción e. etiquetada como defX
  for (k in trabajos){
    cat( sprintf(" defX_%d:", k) )
    
    for (j in trabajos){
      if(j>k || j<k){
        
        for(f in fabricas){
          cat( sprintf(" +X_%d_%d_%d",k,j,f) )          
        }

      }
    }
    cat(" <= 1", "\n")
  }  
  
  
  # Restricción f. etiquetada como defXdummy
  for (f in fabricas){
    cat( sprintf(" defXdummy_%d:", f) )
    
    for (j in trabajos){
      cat( sprintf(" +Xdummy_%d_%d",j,f) )        
    }
    
    cat(" = 1", "\n")
  }   
  
  
  # Restricción g. etiquetada como defprecedencia
  for (k in trabajos){
    if(k<num_trabajos){
      
      for(j in trabajos){
        if(j>k){
          
          cat( sprintf(" defprecedencia_%d_%d: ", k,j))
          for(f in fabricas){
            cat( sprintf(" +X_%d_%d_%d +X_%d_%d_%d",k,j,f,j,k,f) )
          }
          cat(" <= 1", "\n")        
          
        }
      }
    }
  }
    
  
  # Restricción h. etiquetada como defCmax
  for (j in trabajos){
    cat( sprintf(" defCmax_%d: +Cmax - C_%d_%d >= 0", j, j, num_maquinas) )
    cat("\n")
  }
  
  
  # Restricción i.1 etiquetada como defcompletacion0
  for(j in trabajos){
    cat( sprintf(" defcompletacion0_%d: ", j), sprintf(" +C_%d_%d ", j, 1), " >= +", P[j,1], "\n" )
  }

  # Restricción i.2 etiquetada como defcompletacion1
  for(j in trabajos){
    for(i in maquinas){
      if(i >1){
        cat( sprintf(" defcompletacion1_%d_%d: ", j, i), sprintf(" +C_%d_%d -C_%d_%d ", j, i, j, i-1), " >= +", P[j,i], "\n" )
      }
    }
  }

  M <- sum(P)
    
  # Restricción i.3 etiquetada como defcompletacion2
  for(k in trabajos){
    for(j in trabajos){
      if(j>k || j<k){

        for(i in maquinas){
          cat( sprintf(" defcompletacion2_%d_%d_%d: ", k, j, i), sprintf(" +C_%d_%d -C_%d_%d ", j, i, k,i))

          for(f in fabricas){
            cat( " -", M, sprintf(" X_%d_%d_%d ", k,j,f) )
          }
          
          cat(">=", -M+P[j,i], "\n" )
        }
      }
    }
  }
  
  # Bounds
  cat("Bounds", "\n", " Cmax >= 0", "\n")  
  
  for(j in trabajos){
    for(i in maquinas){
     cat( sprintf("C_%d_%d >= 0",j, i), "\n" )
    }
  }

  
  # Binary
  cat("Binary", "\n")  
  
  for(j in trabajos){
    for(f in fabricas){
      
      cat( sprintf("Y_%d_%d",j,f), "\n" )
      cat( sprintf("Xdummy_%d_%d",j,f), "\n" )
      
      for(k in trabajos){
        if(k>j || k<j){
          cat( sprintf("X_%d_%d_%d",k,j,f), "\n" )
        }
      }
      
    }
  }
  
  
  cat("End")
  sink()
}

generacion_modelo_2 <-function(nombre_fichero, num_trabajos, num_fabricas, num_maquinas, d){
  
  crear_fichero(nombre_fichero)
  
  trabajos <- rep(1:num_trabajos)
  maquinas <- rep(1:num_maquinas)
  fabricas <- rep(1:num_fabricas)
  
  # Leyendo Pij
  P<-readRDS( file=sprintf("%d_%d_%d_%d.rds", num_trabajos, num_maquinas, num_fabricas, d) )
  
  # Conexión con el fichero temporal
  sink(nombre_fichero, append=TRUE)
  
  # Restricción b. etiquetada como UnTrabajoUnaFabrica
  for (j in trabajos){
    cat( sprintf(" UnTrabajoUnaFabrica_%d:", j) )
    for (k in trabajos){
      for (f in fabricas){
        cat( sprintf(" +X_%d_%d_%d",j, k, f) )
      }
    }
    cat(" = 1 \n")
  }
  
  # Restricción c. etiquetada como UnaPosicionUnTrabajo
  for (k in trabajos){
    cat( sprintf(" UnaPosicionUnTrabajo_%d:", k) )
    for (j in trabajos){
      for (f in fabricas){
        cat( sprintf(" +X_%d_%d_%d",j, k, f) )
      }
    }
    cat(" = 1 \n")
  }
  
  
  # Restricción d1. etiquetada como RelacionXIW_i1k1f
  if(1<num_trabajos && 1<num_maquinas ){
    for (f in fabricas){
      cat( sprintf(" RelacionXIW_%d_%d_%d:", 1,1,f) )
          
      for(j in trabajos){
            cat(" +", P[j,1], sprintf("X_%d_%d_%d ",j, 2, f) )
      }

      cat( sprintf(" -I_%d_%d_%d +W_%d_%d_%d",2, 1, f, 1, 2, f) )
          
      for(j in trabajos){
            cat(" - ", P[j,2], sprintf("X_%d_%d_%d ",j, 1, f) )
      }
      cat(" = 0", "\n")

    }
  }

  
  # Restricción d2. etiquetada como RelacionXIW_i1
  if(1<num_maquinas){
    for(k in trabajos){
      if(k>1 && k<num_trabajos){
        for (f in fabricas){        
        
          cat( sprintf(" RelacionXIW_%d_%d_%d:", k,1,f) )
          cat( sprintf(" -W_%d_%d_%d",1, k, f) )
          
          for(j in trabajos){
            cat(" +", P[j,1], sprintf("X_%d_%d_%d ",j, k+1, f) )
          }
          
          cat( sprintf("-I_%d_%d_%d +W_%d_%d_%d",2, k, f, 1, k+1, f) )
          
          for(j in trabajos){
            cat(" -", P[j,2], sprintf("X_%d_%d_%d ",j, k, f) )
          }
          
          cat(" = 0", "\n")
        
        }
      }
    }
  }

  
  # Restricción d3. etiquetada como RelacionXIW_k1
  
  if(1<num_trabajos){
    for(i in maquinas){
      if(i>1 && i<num_maquinas){
        for (f in fabricas){        
          
          cat( sprintf(" RelacionXIW_%d_%d_%d:", 1,i,f) )
          cat( sprintf(" +I_%d_%d_%d",i, 1, f) )
          
          for(j in trabajos){
            cat(" +", P[j,i], sprintf("X_%d_%d_%d ",j, 2, f) )
          }
          
          cat( sprintf(" -I_%d_%d_%d +W_%d_%d_%d",i+1, 1, f, i, 2, f) )
          
          for(j in trabajos){
            cat(" -", P[j,i+1], sprintf("X_%d_%d_%d ",j, 1, f) )
          }
          
          cat(" = 0", "\n") 
          
        }
      }
    }
  }
  
  
  # Restricción d4. etiquetada como RelacionXIW
  for (k in trabajos){
    if(k>1 && k<num_trabajos){
      for (i in maquinas){
        if(i<num_maquinas && i>1){
       
          for(f in fabricas){
            
            cat( sprintf(" RelacionXIW_%d_%d_%d:", k,i,f) )
            cat( sprintf(" +I_%d_%d_%d -W_%d_%d_%d ",i, k, f, i, k, f) )
            
            for(j in trabajos){
              cat(" +", P[j,i], sprintf("X_%d_%d_%d ",j, k+1, f) )
            }
            
            cat( sprintf(" -I_%d_%d_%d +W_%d_%d_%d",i+1, k, f, i, k+1, f) )
            
            for(j in trabajos){
              cat(" -", P[j,i+1], sprintf("X_%d_%d_%d ",j, k, f) )
            }
            
            cat(" = 0", "\n")            
            
            
          }
        }
      }
    }
  }
  
  # Restricción e. etiquetada como defCmax
  for (f in fabricas){
    cat( sprintf(" defCmax_%d: +Cmax - C_%d >= 0", f, f) )
    cat("\n")
  }
  
  # Restricción f. etiquetada como defcompletacion
  for (f in fabricas){
    cat( sprintf(" defcompletacion_%d: -C_%d ", f, f) )
    
    for(i in maquinas){
      if(i< num_maquinas){
        for(j in trabajos){
          cat(" +", P[j,i], sprintf("X_%d_%d_%d ",j, 1, f) )
        }
      }
    }
    
    for(k in trabajos){
      if(k<num_trabajos){
        cat(" +", sprintf("I_%d_%d_%d ", num_maquinas, k, f) )
      }
    }
    
    for(k in trabajos){
      for(j in trabajos)
      {
        cat(" +", P[j,num_maquinas], sprintf("X_%d_%d_%d ",j, k, f) )
      }
    }
    cat(" = 0", "\n")
  }
  
  
  # Bounds
  cat("Bounds", "\n", " Cmax >= 0", "\n")  
  
  for(f in fabricas){
    for(i in maquinas){
      for(k in trabajos){
        
        if(k < num_trabajos && i>1){
          cat( sprintf("I_%d_%d_%d >= 0",i, k, f), "\n" )
        }
        
        if(i < num_maquinas && k>1){
          cat( sprintf("W_%d_%d_%d >= 0",i, k, f), "\n" )
        }
        
      }
    }
  }
  

  # Binary
  cat("Binary", "\n")  
  
  for(j in trabajos){
    for(k in trabajos){
      for(f in fabricas){
        cat( sprintf("X_%d_%d_%d",k,j,f), "\n" )
      }
    }
  }
  
  
  
  
  cat("End")
  sink()
}

generacion_modelo_3 <-function(nombre_fichero, num_trabajos, num_fabricas, num_maquinas, d){
  
  crear_fichero(nombre_fichero)
  
  trabajos <- rep(1:num_trabajos)
  maquinas <- rep(1:num_maquinas)
  fabricas <- rep(1:num_fabricas)
  
  # Leyendo Pij
  P<-readRDS( file=sprintf("%d_%d_%d_%d.rds", num_trabajos, num_maquinas, num_fabricas, d) )
  
  # Conexión con el fichero temporal
  sink(nombre_fichero, append=TRUE)
  
  # Restricción b. etiquetada como UnTrabajoUnaFabrica
  for (j in trabajos){
    cat( sprintf(" UnTrabajoUnaFabrica_%d:", j) )
    for (k in trabajos){
      for (f in fabricas){
        cat( sprintf(" +X_%d_%d_%d",j, k, f) )
      }
    }
    cat(" = 1 \n")
  }
  
  # Restricción c. etiquetada como UnaPosicionUnTrabajo
  for (k in trabajos){
    cat( sprintf(" UnaPosicionUnTrabajo_%d:", k) )
    for (j in trabajos){
      for (f in fabricas){
        cat( sprintf(" +X_%d_%d_%d",j, k, f) )
      }
    }
    cat(" = 1 \n")
  }
  
  
  # Restricción d. etiquetada como defCmax
  for (k in trabajos){
    for(f in fabricas){
      cat( sprintf(" defCmax_%d_%d: +Cmax - C_%d_%d_%d >= 0\n", k, f, k, num_maquinas, f) )
    }
  }
  
  # Restricción e.1 etiquetada como defcompletacion0
  for(k in trabajos){
    for(i in maquinas){
      for(f in fabricas){
        
        cat( sprintf(" defcompletacion0_%d_%d_%d: ", k,i,f), sprintf(" +C_%d_%d_%d", k,1,f))
        
        for(j in trabajos){
          cat( "-", P[j,1], sprintf(" X_%d_%d_%d ", j,k,f))
        }
        cat(" >= 0 \n" )
 
      }
    }
  }
  
  
  # Restricción e.2 etiquetada como defcompletacion1
  for(k in trabajos){
    for(i in maquinas){
      if(i>1){
        
        for(f in fabricas){
          
          cat( sprintf(" defcompletacion1_%d_%d_%d: ", k,i,f), sprintf(" +C_%d_%d_%d -C_%d_%d_%d", k,i,f, k, i-1, f))
          
          for(j in trabajos){
            cat( "-", P[j,i], sprintf(" X_%d_%d_%d ", j,k,f))
          }
          cat(" >= 0 \n" )
          
        }
      }
    }
  }
  
  # Restricción e.3 etiquetada como defcompletacion2
  for(k in trabajos){
    if(k>1){
      
      for(i in maquinas){
        for(f in fabricas){
          
          cat( sprintf(" defcompletacion2_%d_%d_%d: ", k,i,f), sprintf(" +C_%d_%d_%d -C_%d_%d_%d", k,i,f, k-1, i, f))
          
          for(j in trabajos){
            cat( "-", P[j,i], sprintf(" X_%d_%d_%d ", j,k,f))
          }
          cat(" >= 0 \n" )
          
        }
      }
    }
  } 
  
  
  # Bounds
  cat("Bounds", "\n", " Cmax >= 0", "\n")  
  
  for(k in trabajos){
    for(i in maquinas){
      for(f in fabricas){
        cat( sprintf("C_%d_%d_%d >= 0",k,i,f), "\n" )
      }
    }
  }
  

  # Binary
  cat("Binary", "\n")  
  
  for(j in trabajos){
    for(k in trabajos){
      for(f in fabricas){
        cat( sprintf("X_%d_%d_%d",j,k,f), "\n" )
      }
    }
  }
  
  cat("End")
  sink()
}

generacion_modelo_4 <-function(nombre_fichero, num_trabajos, num_fabricas, num_maquinas, d){
  
  crear_fichero(nombre_fichero)
  
  trabajos <- rep(1:num_trabajos)
  maquinas <- rep(1:num_maquinas)
  fabricas <- rep(1:num_fabricas)
  
  # Leyendo Pij
  P<-readRDS( file=sprintf("%d_%d_%d_%d.rds", num_trabajos, num_maquinas, num_fabricas, d) )
  
  # Conexión con el fichero temporal
  sink(nombre_fichero, append=TRUE)
  
  # Restricción b. etiquetada como UnTrabajoUnaPosicion
  for (j in trabajos){
    cat( sprintf(" UnTrabajoUnaPosicion_%d:", j) )
    for (k in trabajos){
      cat( sprintf(" +X_%d_%d",j, k) )
    }
    cat(" = 1 \n")
  }
  
  # Restricción c. etiquetada como UnaPosicionUnTrabajo
  for (k in trabajos){
    cat( sprintf(" UnaPosicionUnTrabajo_%d:", k) )
    
    for (j in trabajos){
      cat( sprintf(" +X_%d_%d",j, k) )       
    }
    cat(" = 1 \n")
  }
  
  
  # Restricción d. etiquetada como UnTrabajoUnaFabrica
  for (k in trabajos){
    cat( sprintf(" UnTrabajoUnaFabrica_%d:", k) )
    for (f in fabricas){
      cat( sprintf(" +Y_%d_%d",k, f) )
    }
    cat(" = 1 \n")
  }

  
  # Restricción e. etiquetada como defCmax
  for (k in trabajos){
    cat( sprintf(" defCmax_%d: +Cmax - C_%d_%d >= 0", k, k, num_maquinas) )
    cat("\n")
  }
  
  
  # Restricción f.1 etiquetada como defcompletacion0
  for(k in trabajos){
    cat( sprintf(" defcompletacion0_%d: ", k), sprintf(" +C_%d_%d ", k, 1))
    
    for(j in trabajos){
      cat( "-", P[j,1], sprintf(" X_%d_%d ", j,k))
    }
    cat(" >= 0 \n" )
  }
  
  
  # Restricción f.2 etiquetada como defcompletacion1
  for(k in trabajos){
    for(i in maquinas){
      if(i>1){

        cat( sprintf(" defcompletacion1_%d_%d: ", k,i), sprintf(" +C_%d_%d -C_%d_%d", k,i, k, i-1))
          
        for(j in trabajos){
          cat( " -", P[j,i], sprintf(" X_%d_%d ", j,k))
        }
        cat(" >= 0 \n" )
          
      }
    }
  }
  
  
  M <- sum(P)
  
  # Restricción f.3 etiquetada como defcompletacion2
  for(k in trabajos){
    if(k>1){
      
      for(l in trabajos){
        if(l<k){
          
          for(i in maquinas){
            for(f in fabricas){
              
              cat( sprintf(" defcompletacion2_%d_%d_%d_%d: ", k,l,i,f), sprintf(" +C_%d_%d -C_%d_%d", k,i,l,i))
              
              for(j in trabajos){
                cat( " -", P[j,i], sprintf(" X_%d_%d ", j,k))
              }
              
              cat( " -", M, sprintf(" Y_%d_%d", k,f), " -", M, sprintf(" Y_%d_%d", l,f))
                   
              cat(" >= ", -2*M, "\n" )
              
            }
          }
        }
      }
    }
  } 
  
  
  # Bounds
  cat("Bounds", "\n", " Cmax >= 0", "\n")  
  
  for(k in trabajos){
    for(i in maquinas){
      cat( sprintf("C_%d_%d >= 0",k, i), "\n" )
    }
  }
  
  
  # Binary
  cat("Binary", "\n")  
  
  
  for(k in trabajos){
    
    for(j in trabajos){
      cat( sprintf("X_%d_%d",j,k), "\n" )
    }
    
    for(f in fabricas){
      cat( sprintf("Y_%d_%d",k,f), "\n" )
    }
  }

  
  
  
  cat("End")
  sink()
}

generacion_modelo_5 <-function(nombre_fichero, num_trabajos, num_fabricas, num_maquinas, d){
  
  crear_fichero(nombre_fichero)
  
  trabajos <- rep(1:num_trabajos)
  maquinas <- rep(1:num_maquinas)
  fabricas <- rep(1:num_fabricas)
  
  # Leyendo Pij
  P<-readRDS( file=sprintf("%d_%d_%d_%d.rds", num_trabajos, num_maquinas, num_fabricas, d) )
  
  # Conexión con el fichero temporal
  sink(nombre_fichero, append=TRUE)
  
  # Restricción b. etiquetada como UnTrabajoUnaPosicion
  for (j in trabajos){
    cat( sprintf(" UnTrabajoUnaPosicion_%d: +X_kdummy_%d", j, j) )
    
    for (k in trabajos){
      if(j>k || j<k){
        cat( sprintf(" +X_%d_%d",k,j) )        
      }
    }
    cat(" = 1 \n")
  }
  
  # Restricción c. etiquetada como DefX
  for (k in trabajos){
    cat( sprintf(" DefX_%d: +X_jdummy_%d", k,k) )
  
     for (j in trabajos){
       if(j>k || j<k){
         cat( sprintf(" +X_%d_%d",k, j) )         
        }
    }
    cat(" <= 1 \n")
  }
  
  
  # Restricción d.1 etiquetada como Separadores1
  cat(" Separadores1: ")
  for (j in trabajos){
    cat( sprintf(" +X_kdummy_%d", j) )
  }
  cat(" = ", num_fabricas, "\n")
  
  
  # Restricción d.2 etiquetada como Separadores2
  cat(" Separadores2: ")
  for (k in trabajos){
    cat( sprintf(" +X_jdummy_%d", k) )
  }
  cat(" = ", num_fabricas-1, "\n")  
  
 
  
  # Restricción e. etiquetada como defprecedencia
  for (k in trabajos){
    if(k<num_trabajos){
      for(j in trabajos){
        if(j>k){
          cat( sprintf(" defprecedencia_%d_%d: ",k,j), sprintf(" +X_%d_%d +X_%d_%d",k,j,j,k), " <= 1\n" )
        }
      }
    }
  }
  
  
  # Restricción f. etiquetada como defCmax
  for (j in trabajos){
    cat( sprintf(" defCmax_%d: +Cmax - C_%d_%d >= 0\n", j, j, num_maquinas) )
  }
  
  
  # Restricción i.1 etiquetada como defcompletacion0
  for(j in trabajos){
    cat( sprintf(" defcompletacion0_%d: ", j), sprintf(" +C_%d_%d ", j, 1), " >= +", P[j,1], "\n" )
  }
  
  
  # Restricción i.2 etiquetada como defcompletacion1
  for(j in trabajos){
    for(i in maquinas){
      if(i >1){
        cat( sprintf(" defcompletacion1_%d_%d: ", j, i), sprintf(" +C_%d_%d -C_%d_%d ", j, i, j, i-1), " >= +", P[j,i], "\n" )
      }
    }
  }
  
  M <- sum(P)
  
  # Restricción i.3 etiquetada como defcompletacion2
  for(j in trabajos){
    for(i in maquinas){
      cat( sprintf(" defcompletacion2_%d_%d: ", j, i), sprintf(" +C_%d_%d ", j, i), -M, sprintf(" X_kdummy_%d", j)   )
      cat(">= ", -M+P[j,i], "\n" )
    }
  } 
  
 
  # Restricción i.4 etiquetada como defcompletacion3
  for(k in trabajos){
    for(j in trabajos){
      for(i in maquinas){
        if(j>k || j<k){
          
          cat( sprintf(" defcompletacion3_%d_%d_%d: ", k, j, i), sprintf(" +C_%d_%d -C_%d_%d ", j, i, k,i))
          cat( " -", M, sprintf(" X_%d_%d", k,j) )
          cat(">=", -M+P[j,i], "\n" )
        }
      }
    }
  }
  
  
  # Bounds
  cat("Bounds", "\n", " Cmax >= 0", "\n")  
  
  for(j in trabajos){
    for(i in maquinas){
      cat( sprintf("C_%d_%d >= 0",j, i), "\n" )
    }
  }
  
  
  # Binary
  cat("Binary", "\n")  
  
  for(j in trabajos){
    
    cat( sprintf("X_kdummy_%d",j), "\n" )
    
    for(k in trabajos){
      
      if(j==1){
        cat( sprintf("X_jdummy_%d",k), "\n" )
      }
      
      if(k>j || k<j){
        cat( sprintf("X_%d_%d",k,j), "\n" )
      }
      
    }
  }
  
  
  cat("End")
  sink()
}

generacion_modelo_6 <-function(nombre_fichero, num_trabajos, num_fabricas, num_maquinas, d){
  
  crear_fichero(nombre_fichero)
  
  trabajos <- rep(1:num_trabajos)
  maquinas <- rep(1:num_maquinas)
  fabricas <- rep(1:num_fabricas)
  
  # Leyendo Pij
  P<-readRDS( file=sprintf("%d_%d_%d_%d.rds", num_trabajos, num_maquinas, num_fabricas, d) )
  
  # Conexión con el fichero temporal
  sink(nombre_fichero, append=TRUE)
  
  # Restricción b. etiquetada como UnTrabajoUnaFabrica
  for (j in trabajos){
    cat( sprintf(" UnTrabajoUnaFabrica_%d:", j) )
    for (f in fabricas){
      cat( sprintf(" +Y_%d_%d",j, f) )
    }
    cat(" = 1 \n")
  }

  # Restricción b. etiquetada como defCmax
  for (j in trabajos){
    cat( sprintf(" defCmax_%d: +Cmax - C_%d_%d >= 0\n", j, j, num_maquinas) )
  }
  
  
  # Restricción c.1 etiquetada como defcompletacion0
  for(j in trabajos){
    cat( sprintf(" defcompletacion0_%d: ", j), sprintf(" +C_%d_%d ", j, 1), " >= +", P[j,1], "\n" )
  }
  
  # Restricción c.2 etiquetada como defcompletacion1
  for(j in trabajos){
    for(i in maquinas){
      if(i >1){
        cat( sprintf(" defcompletacion1_%d_%d: ", j, i), sprintf(" +C_%d_%d -C_%d_%d ", j, i, j, i-1), " >= +", P[j,i], "\n" )
      }
    }
  }
  
  M <- sum(P)
  
  # Restricción c.3 y c.4. etiquetadas como defcompletacion2 y defcompletacion3
  for(k in trabajos){
    if(k < num_trabajos){
      
      for(j in trabajos){
        if(j>k){
          
          for(i in maquinas){
            for(f in fabricas){
              
              cat( sprintf(" defcompletacion2_%d_%d_%d_%d: ", k, j, i, f), sprintf(" +C_%d_%d -C_%d_%d ", k, i, j,i))
              cat( " +", M, sprintf(" X_%d_%d ", k,j) )
              cat( " -", M, sprintf(" Y_%d_%d ", k,f) )
              cat( " -", M, sprintf(" Y_%d_%d ", j,f) )
              cat(">=", -2*M+P[k,i], "\n" )
              
              cat( sprintf(" defcompletacion3_%d_%d_%d_%d: ", k, j, i, f), sprintf(" +C_%d_%d -C_%d_%d ", j, i, k,i))
              cat( " -", M, sprintf(" X_%d_%d ", k,j) )
              cat( " -", M, sprintf(" Y_%d_%d ", k,f) )
              cat( " -", M, sprintf(" Y_%d_%d ", j,f) )
              cat(">=", -3*M+P[j,i], "\n" )

            }
          }
        }
      }
    }
  }
    
  
  
  # Bounds
  cat("Bounds", "\n", " Cmax >= 0", "\n")  
  
  for(j in trabajos){
    for(i in maquinas){
      cat( sprintf("C_%d_%d >= 0",j, i), "\n" )
    }
  }
  
  
  # Binary
  cat("Binary", "\n")  
  
  for(j in trabajos){
    
    for(f in fabricas){
      cat( sprintf("Y_%d_%d",j,f), "\n" )
    }
    
    for(k in trabajos){
      if(k<j && k<num_trabajos){
        cat( sprintf("X_%d_%d",k,j), "\n" )
      }
    }
    
  }
  
  
  cat("End")
  sink()
}

